{% extends "postorius/base.html" %}
{% load i18n %}
{% load bootstrap_tags %}

{% block head_title %}
{% trans 'Add domain owner ' %} - {{ block.super }}
{% endblock %}

{% block content %}

    <div class="page-header">
        <h2>{% trans 'Add a new owner to' %} {{ domain }}</h2>
    </div>

    <form action="{% url 'domain_owners' domain.mail_host %}" method="post" class="form-horizontal">
        {% bootstrap_form_horizontal form 2 8 'Add Owner' %}
    </form>

{% endblock content %}
