# Generated by Django 2.0.6 on 2018-07-11 20:59

from django.db import migrations, models


# flake8: noqa


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0005_auto_20180707_1107'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='data',
            field=models.TextField(help_text="Note: Do not add any secret content in templates as they are publicly accessible.\nYou can use these variables in the templates. \n$hyperkitty_url: Permalink to archived message in Hyperkitty\n$listname: Name of the Mailing List e.g. ant@example.com \n$list_id: The List-ID header e.g. ant.example.com \n$display_name: Display name of the mailing list e.g. Ant \n$short_listname: Local part of the listname e.g. ant \n$domain: The domain part of the listname e.g. example.com \n$info: The mailing list's longer descriptive text \n$request_email: The email address for -request address \n$owner_email: The email address for -owner address \n$site_email: The email address to reach the owners of the site \n$language: The two letter language code for list's preferred language e.g. fr, en, de \n"),
        ),
    ]
