# Generated by Django 3.2.12 on 2022-03-04 13:12

from django.db import migrations, models


# flake8: noqa

class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0017_alter_emailtemplate_language'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='language',
            field=models.CharField(blank=True, choices=[('ar', 'Arabic'), ('ast', 'Asturian'), ('bg', 'Bulgarian'), ('ca', 'Catalan'), ('zh_CN', 'Chinese'), ('zh_TW', 'Chinese (Taiwan)'), ('hr', 'Croatian'), ('cs', 'Czech'), ('da', 'Danish'), ('nl', 'Dutch'), ('en', 'English (USA)'), ('et', 'Estonian'), ('eu', 'Euskara'), ('fi', 'Finnish'), ('fr', 'French'), ('gl', 'Galician'), ('de', 'German'), ('el', 'Greek'), ('he', 'Hebrew'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('it', 'Italian'), ('ja', 'Japanese'), ('ko', 'Korean'), ('lt', 'Lithuanian'), ('no', 'Norwegian'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt_BR', 'Portuguese (Brazil)'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sr', 'Serbian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('es', 'Spanish'), ('sv', 'Swedish'), ('tr', 'Turkish'), ('uk', 'Ukrainian'), ('vi', 'Vietnamese')], help_text="Language for the template, this should be the list's preferred language.", max_length=5),
        ),
    ]
