{% extends "postorius/base.html" %}
{% load i18n %}

{% block content %}
    <h1>{% trans 'Page not found' %}</h1>
    <div class="alert alert-danger">{% trans "This page either doesn't exist, or it moved somewhere else." %}</div>
{% endblock %}
