# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-13 09:48


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0002_auto_20160210_0721'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='addressconfirmationprofile',
            name='user',
        ),
        migrations.DeleteModel(
            name='AddressConfirmationProfile',
        ),
    ]
