{% extends "postorius/base.html" %}
{% load i18n %}
{% load bootstrap_tags %}
{% load nav_helpers %}

{% block head_title %}
{% trans 'Mass subscribe' %} | {{ list.fqdn_listname }} - {{ block.super }}
{% endblock %}

{% block content %}

    {% list_nav 'list_mass_ops' 'msgid:title:list_mass_subscription' %}

    <form action="{% url 'mass_subscribe' list.list_id %}" method="post" class="form-horizontal">
        {% bootstrap_form_horizontal form 2 8 'Subscribe users' %}
        <p hidden>{% trans 'Subscribe users' %}</p>
    </form>

{% endblock content %}
