{% extends "postorius/base.html" %}
{% load i18n %}

{% block content %}
    <h1>{% trans 'Forbidden' %}</h1>
    <div class="alert alert-danger">{% trans "You do not have permissions to visit this page." %}</div>
{% endblock %}
