{% extends "postorius/base.html" %}
{% load i18n %}

{% block head_title %}
{% trans 'Global Bans' %} - {{ block.super }}
{% endblock %}

{% block content %}
<header class="page-header">
    {% trans "Global Bans" %}
</header>

{% url 'global_bans' as formurl %}
{% include 'postorius/fragments/bans.html' with formurl=formurl %}

{% endblock content %}
